#ifndef BOUNDING_VOLUME_HIERARCHY_H
#define BOUNDING_VOLUME_HIERARCHY_H

#include "trax.hpp"
#include "Box.h"
#include "Triangle.h"

class BoundingVolumeHierarchy {
 public:
  
  BoundingVolumeHierarchy(const int &_start_scene)
    {
      start_bvh = _start_scene;
    }

  inline void intersect(HitRecord& hit, const Ray& ray) const
  {
    int stack[32];
    int node_id = 0;
    int sp = 0;
    while(true)
    {
      int node_addr = start_bvh + node_id * 8;
      Box b = loadBoxFromMemory(node_addr);
      HitRecord boxHit;
      b.intersect(boxHit, ray);
      if(boxHit.didHit())
	{
	  // only traverse the node if we haven't found something closer
	  if(boxHit.didHitInside() || (hit.minT() >= boxHit.minT())) 
	    {
	      node_id = loadi( node_addr, 7 );
	      int num_children = loadi( node_addr, 6 );
	      if ( num_children < 0 ) // interior node
		{
		  stack[ sp++ ] = node_id + 1;
		  continue;
		}
	      // leaf node
	      node_addr = node_id;
	      for ( int i = 0; i < num_children; ++i)
		{
		  Tri t = loadTriFromMemory(node_addr);
		  t.intersect(hit, ray);
		  node_addr = node_addr + 11;
		}
	    }
	}
      if ( --sp < 0 )
	{
	  return;
	}
      node_id = stack[ sp ];
    }
  }

 private:
  int start_bvh;
  
};
#endif
