#include "Triangle.h"
#include "PointLight.h"
#include "BoundingVolumeHierarchy.h"
#include "Vector.h"
#include "trax.hpp"

inline float fabs(float const &f){
  if(f>0.0f)
    { return f; }
  return f * -1.0f;
}

Vector getOrtho(Vector const &normal)
{
  Vector axis(0.0f, 0.0f, 0.0f);
  float ax = fabs(normal.X);
  float ay = fabs(normal.Y);
  float az = fabs(normal.Z);

  if(ax < ay && ax < az)
    { axis = Vector(1.0f, 0.0f, 0.0f); }
  else if (ay < az)
    { axis = Vector(0.0f, 1.0f, 0.0f); }
  else
    { axis = Vector(0.0f, 0.0f, 1.0f); }
  return Cross(normal, axis);
}


Vector randomReflection(const Vector &normal)
{

  Vector refDir(0.0f, 0.0f, 0.0f);
  // pick random point on disk [-1, 1]
  float x = 0.0f;
  float y = 0.0f;
  float z = 0.0f;
  float x_2 = 0.0f; // squares
  float y_2 = 0.0f;
  do 
    {
      x = trax_rand(); 
      y = trax_rand();
      x = x * 2.0f;
      x = x - 1.0f;
      y = y * 2.0f;
      y = y - 1.0f;
      x_2 = x * x;
      y_2 = y * y;
    }
  while((x_2 + y_2) >= 1.0f); // cut out points outside the disk
  
  z = sqrt(1.0f - x_2 - y_2); 
  Vector X = getOrtho(normal);
  Vector Y = Cross(normal, X);

  refDir = (X * x) + (Y * y) + (normal * z);
  refDir.normalize();
  
  return refDir;
}

Color shade(const HitRecord &hr, const Ray &ray, const BoundingVolumeHierarchy &bvh, const PointLight &pl, const int &start_matls, Vector &hit_point, Vector &normal, Color &hit_color)
{
  Color background = Color(0.561f, 0.729f, 0.988f); // hard coded for run-rt
  //alternatively, load the background from trax memory

  //float epsilon = loadf(0, 18);
  float epsilon = 0.001f;

  if(hr.didHit())
    {
      hit_point = ray.origin() + (hr.minT() * ray.direction());
      Tri hit_tri = loadTriFromMemory(hr.getID());
      normal = hit_tri.normal();
      float costheta = Dot(normal, ray.direction());
      if(costheta > 0.0f)
	normal = -normal;

      hit_point = hit_point + (normal * epsilon); // offset to prevent self-shadowing

      Color retVal(0.f, 0.f, 0.f);

      for(int i=0; i < 1; i++) // hard-code for now for a single point-light source
	{
	  Color lightColor;
	  Vector lightDir;
	  float lightDistance = pl.getLight(lightColor, lightDir, hit_point);
	  float cosphi = Dot(normal, lightDir);
	  if(cosphi > 0.0f)
	    {
	      Ray shadow(hit_point, lightDir);
	      HitRecord shadowHit(lightDistance);
	      
	      bvh.intersect(shadowHit, shadow);

	      // if the shadow ray doesn't intersect the scene
	      if(!shadowHit.didHit())
		{ 
		  retVal += (lightColor * cosphi * 0.7f);
		}
	    }
	}
      int shader = loadi(hr.getID() + 10);
      shader = start_matls + shader * 25; // mats are 25 words
      hit_color = loadColorFromMemory(shader);

      retVal *= hit_color;
      
      return retVal;
    }
  else
  
    return background;
}


