#ifndef PinholeCamera_h
#define PinholeCamera_h

#include "Vector.h"
#include "Ray.h"

class PinholeCamera{
 public:
  PinholeCamera()
    {}
  PinholeCamera(int addr);
  PinholeCamera(const Vector& eye, const Vector& lookat, const Vector& up, float ulen, float aspect_ratio);
  void makeRay(Ray& ray, float x, float y) const;

 private:
  Vector eye;
  Vector lookat;
  Vector up;
  float ulen;
  float aspect_ratio;

  Vector u;
  Vector v;
  Vector lookdir;
};

inline PinholeCamera::PinholeCamera(int addr)
{
  eye = loadVectorFromMemory(addr);
  up = loadVectorFromMemory(addr + 9);
  lookdir = loadVectorFromMemory(addr + 12);
  u = loadVectorFromMemory(addr + 15);
  v = loadVectorFromMemory(addr + 18);
}

inline PinholeCamera::PinholeCamera(const Vector& eye, const Vector& lookat, const Vector& up,
                             float ulen, float aspect_ratio)
: eye(eye), lookat(lookat), up(up), ulen(ulen), aspect_ratio(aspect_ratio)
{
  lookdir = lookat-eye;
  lookdir.normalize();
  u = Cross(lookdir, up);
  v = Cross(u, lookdir);
  //double ulen = tan(hfov/2.*M_PI/180.);
  u.normalize();
  u *= ulen;
  float vlen = ulen/aspect_ratio;
  v.normalize(); 
  v *= vlen;
}

inline void PinholeCamera::makeRay(Ray& ray, float x, float y) const
{
  Vector direction = lookdir+u*x+v*y;
  direction.normalize();
  ray = Ray(eye, direction);
}

#endif
